From 045c10de94425fba6ba68a123cc12077add7e89d Mon Sep 17 00:00:00 2001
From: Wang Shengfang <wangshengfang@huawei.com>
Date: Fri, 3 Apr 2020 15:21:10 +0800
Subject: [PATCH] utils_test: Add new_xml_with_object() to avoid triggering the
 GC and deleting the xml file

Add the xml object reference to the function return value
to avoid triggering the GC and deleting the xml file
after calling the create xml function.

Signed-off-by: Wang Shengfang <wangshengfang@huawei.com>
Signed-off-by: Chen Qun <kuhn.chenqun@huawei.com>
---
 virttest/utils_test/libvirt.py | 39 +++++++++++++++++++++++-----------
 1 file changed, 27 insertions(+), 12 deletions(-)

diff --git a/virttest/utils_test/libvirt.py b/virttest/utils_test/libvirt.py
index 5d8891b5..b5e2132f 100644
--- a/virttest/utils_test/libvirt.py
+++ b/virttest/utils_test/libvirt.py
@@ -167,6 +167,28 @@ class LibvirtNetwork(object):
             virsh.net_undefine(self.name)
 
 
+class TempStr(str):
+    """
+    String class for new_xml_with_object.
+    """
+    pass
+
+
+def new_xml_with_object(xml, obj):
+    """
+    Add the xml object reference to the function return value
+    to avoid triggering the GC and deleting the xml file
+    after calling the create xml function.
+
+    :param xml: xml path string
+    :param obj: xml object reference
+    :return: xml path string
+    """
+    _temp = TempStr(xml)
+    _temp.obj = obj
+    return _temp
+
+
 def create_macvtap_vmxml(iface, params):
     """
     Method to create Macvtap interface xml for VM
@@ -1766,7 +1788,7 @@ def create_hostdev_xml(pci_id, boot_order=None, xmlfile=True,
     logging.debug("Hostdev XML:\n%s", str(hostdev_xml))
     if not xmlfile:
         return hostdev_xml
-    return hostdev_xml.xml
+    return new_xml_with_object(hostdev_xml.xml, hostdev_xml)
 
 
 def create_controller_xml(contr_dict, oper="get_xml", vm_name=None):
@@ -1809,7 +1831,7 @@ def create_controller_xml(contr_dict, oper="get_xml", vm_name=None):
             raise exceptions.TestFail("Fail to get new added controller.")
         return new_added_ctr
     elif oper == "get_xml":
-        return contr.xml
+        return new_xml_with_object(contr.xml, contr)
 
 
 def create_redirdev_xml(redir_type="spicevmc", redir_bus="usb",
@@ -1836,7 +1858,7 @@ def create_redirdev_xml(redir_type="spicevmc", redir_bus="usb",
     if redir_protocol:
         redir.protocol = eval(redir_protocol)
 
-    return redir.xml
+    return new_xml_with_object(redir.xml, redir)
 
 
 def alter_boot_order(vm_name, pci_id, boot_order=0):
@@ -2010,14 +2032,7 @@ def create_disk_xml(params):
         logging.error("Fail to create disk XML:\n%s", detail)
     logging.debug("Disk XML %s:\n%s", diskxml.xml, str(diskxml))
 
-    # Wait for file completed
-    def file_exists():
-        if not process.run("ls %s" % diskxml.xml,
-                           ignore_status=True).exit_status:
-            return True
-    utils_misc.wait_for(file_exists, 5)
-
-    return diskxml.xml
+    return new_xml_with_object(diskxml.xml, diskxml)
 
 
 def set_disk_attr(vmxml, target, tag, attr):
@@ -3724,7 +3739,7 @@ def modify_vm_iface(vm_name, oper, iface_dict, index=0):
         vmxml.sync()
     elif oper == "get_xml":
         logging.info("iface xml is %s", iface)
-        return iface.xml
+        return new_xml_with_object(iface.xml, iface)
 
 
 def change_boot_order(vm_name, device_tag, boot_order, index=0):
-- 
2.23.0

